#!/usr/local/bin/perl
#
# flow-smurf.pl, 1.3
# Stephen Sprunk <ssprunk@cisco.com>
# Ben Black <black@layer8.net>
# (c)1999  All rights reserved.
#
# input: sorted output from "sh ip cache flow"
# output: list of probable smurf amplifiers
#
# Command line options:
#   -p n    Prefix length for grouping addresses (default 24)
#   -s n    Minimum ICMP size to consider (default 100 bytes)
#   -f n    Minimum flows per prefix (default 3)
#   -b n    Minimum bytes per prefix (default 1000000)
#
# If minimum flows OR minimum bytes is exceeded, the flow is marked
#

use Socket, Getopt::Std;

getopt('p:s:f:b:');
$prefix = $opt_p ? $opt_p : 24;
$minsize = $opt_s ? $opt_s : 100;
$minflows = $opt_f ? $opt_f : 3;
$minbytes = $opt_b ? $opt_b : 1000000;
$mask = 0xffffffff >> (32-$prefix) << (32-$prefix);

while (<>) {

    s/  */ /;
    ($srcif, $srcip, $dstif, $dstip, $pr, $srcp, $dstp, $pkts, $bppk, $active) =
      split;

    $srcnet = (unpack N,inet_aton($srcip)) & $mask;

    next unless (($pr == 1) && ($bppk >= $minsize));

    if ($oldnet != $srcnet) {
	$flows = 0;
	$bytes = 0;
	$printed = 0;
	$oldnet = $srcnet;
    }

    $flows++;
    $bytes += $pkts * $bppk;

    next unless (($flows >= $minflows) || ($bytes >= $minbytes));

    print inet_ntoa(pack N, $srcnet) . "\n" unless $printed;
    $printed = 1;
}

